/**
 * grid-menu.js
 *
 * @author     James Brumond
 * @version    0.0.1
 * @copyright  Copyright 2011 James Brumond
 * @license    Dual licensed under MIT and GPL
 */

var GridMenu = (function() {
	
	// Default configuration settings for GridMenu
	var _defaultConfig = {
		rows: 0,
		cols: 0,
		width: null,
		height: null,
		items: [ ],
		strategy: 'alignStrict',
		dragDelay: 1000
	};
	
	// Default item values
	var _defaultItem = {
		contents: null,
		onselect: null,
		ondragstart: null,
		ondrag: null,
		ondragend: null
	};
	
	// Is this a touch device?
	var touchEvents = (function() {
		var ret, elem = document.createElement('div');
		ret = ('ontouchstart' in elem);
		elem = null;
		return ret;
	}());
	
	// A class to add when an element is being dragged
	var dragClass = 'drag';
	
	// Determine the events to bind to
	var events = (touchEvents ?
		{
			start: 'touchstart',
			move: 'touchmove',
			end: 'touchend'
		} : {
			start: 'mousedown',
			move: 'mousemove',
			end: 'mouseup'
		}
	);

// ------------------------------------------------------------------
//  GridMenu Constructor

	/**
	 * GridMenu Constructor
	 *
	 * @access  public
	 * @param   object    configuration options
	 */
	function GridMenu(conf) {
		this._objectId = objectId();
		this._conf = merge({ }, _defaultConfig, conf);
		// Make sure the strategy exists
		if (typeof Grid._strategies[this._conf.strategy] !== 'function') {
			throw 'Invalid grid restructure strategy "' + this._conf.strategy + '"';
		}
		// Check/clone the item objects
		var items = this._conf.items;
		for (var i = 0, c = items.length; i < c; i++) {
			if (! isObject(items[i])) {
				items[i] = { contents: items[i] };
			}
			items[i] = merge({ id: i }, _defaultItem, items[i]);
		}
		// Build the layout grid
		this._grid = new Grid(this);
		// Build the DOM structure
		this._create();
	}
	
	/**
	 * Make any changes in the layout object
	 *
	 * @access  public
	 * @return  void
	 */
	GridMenu.prototype.redraw = function() {
		var items = this._conf.items;
		// Pull all the item elements
		for (var i = 0, c = items.length; i < c; i++) {
			items[i].node.parentNode.removeChild(items[i].node);
		}
		// Put the items back in in the correct order
		var layout = this._grid._buildItemList();
		var lis = this.domNode.getElementsByTagName('li');
		for (var item, i = 0, c = layout.length; i < c; i++) {
			if (typeof layout[i] !== 'number') {continue;}
			item = items[layout[i]];
			item.node.style.top = '0px';
			item.node.style.left = '0px';
			lis[i].appendChild(item.node);
		}
	};
	
	/**
	 * Create the item elements and the DOM structure
	 *
	 * @access  public
	 * @return  void
	 */
	GridMenu.prototype._create = function() {
		// We need this in the event binding later...
		var self = this;
		// Create the root element
		var root = createElement('ul', {
			id: 'grid-menu-' + this._objectId,
			style: {
				position: 'relative',
				width: this._conf.width + 'px',
				height: this._conf.height + 'px'
			}
		});
		// Prepare to build the items
		var items = this._conf.items;
		var itemDimensions = {
			x: this._conf.width / this._conf.cols,
			y: this._conf.height / this._conf.rows
		};
		var currentLi;
		for (var row = 0, item = 0; row < this._conf.rows; row++) {
			for (var col = 0; col < this._conf.cols; col++, item++) {
				// Build the item's LI
				currentLi = root.appendChild(
					createElement('li', {
						style: {
							position: 'absolute',
							width: itemDimensions.x + 'px',
							height: itemDimensions.y + 'px',
							top: (itemDimensions.y * row) + 'px',
							left: (itemDimensions.x * col) + 'px'
						}
					})
				);
				// Build the item's DIV
				items[item].node = currentLi.appendChild(
					createElement('div', {
						_contents: items[item].contents,
						className: 'grid-menu-item',
						id: 'grid-menu-item-' + objectId(),
						style: {
							top: '0px',
							left: '0px',
							position: 'relative',
							width: itemDimensions.x + 'px',
							height: itemDimensions.y + 'px'
						}
					})
				);
				// Create a closure to hold on to a reference to the node
				(function(element, item) {
					// Bind the initializing event
					element._menuEvent = Events.bind(element, events.start, function(e) {
						// Make sure that if this is a mouse event, that the left mouse
						// button is what was clicked. (this structure is kind of ugly
						// with 3 nested ifs, but its the best way I can think to write
						// it, so just live with it...)
						if (e.type === 'mousedown') {
							if (window.event === e) {
								if (e.button === 1) {return false;}
							} else if (e.button === 0) {return false;}
						}
						var releaseTimer;
						var releaseEvent;
						// Releases both the event and timer
						function release() {
							releaseEvent();
							clearTimeout(releaseTimer);
							textSelection.enable();
						}
						// Start the drag timer/flag
						releaseTimer = setTimeout(function() {
							// If the strategy value is "none", don't allow rearrangement
							if (self._conf.strategy !== 'none') {
								release();
								self._dragItem(item, e);
							}
						}, self._conf.dragDelay);
						// Bind the release event
						releaseEvent = Events.bind(document, events.end, function(e) {
							release();
							self._raiseEvent('select', item, e);
						});
						// Avoid text selection problems
						textSelection.disable(element);
					});
				}(items[item].node, items[item]));
			}
		}
		// Expose the top-level DOM node
		this.domNode = root;
	};
	
	/**
	 * This function is called when an item is selected (quick tap)
	 *
	 * @access  public
	 * @param   string    the event type
	 * @param   object    the item object
	 * @param   object    the event object
	 * @return  void
	 */
	GridMenu.prototype._raiseEvent = function(type, item, event) {
		if (typeof item['on' + type] === 'function') {
			item['on' + type].call(this,
				new GridMenuEvent(type, event, { item: item })
			);
		}
	};
	
	/**
	 * This initializes the item dragging
	 *
	 * @access  public
	 * @param   object    the item object
	 * @param   object    the event object
	 * @return  void
	 */
	GridMenu.prototype._dragItem = function(item, e) {
		var self = this;
		var element = item.node;
		// Add the dragging class to the DIV
		addClass(element, dragClass);
		// Store a reference to the current position
		var startX = getPos(element, 'left');
		var startY = getPos(element, 'top');
		// Determine the bounding box
		var liNode = element.parentNode;
		var minX = -getPos(liNode, 'left');
		var maxX = minX + self._conf.width;
		var minY = -getPos(liNode, 'top');
		var maxY = minY + self._conf.height;
		// We need to know the last position when we release
		var lastPosition = null;
		// Bind the drag event
		var tempEvents = [ ];
		tempEvents.push(
			Events.bind(document, events.move, function(e2) {
				// Find all needed offsets
				var offsetX = e2.clientX - e.clientX;
				var offsetY = e2.clientY - e.clientY;
				var offsetWidth = element.offsetWidth;
				var offsetHeight = element.offsetHeight;
				// Find the new positions
				var posX = startX + offsetX;
				var posY = startY + offsetY;
				// Enforce the bounding box
				posX = Math.max(minX, Math.min(maxX - offsetWidth, posX));
				posY = Math.max(minY, Math.min(maxY - offsetHeight, posY));
				// Store the position
				lastPosition = { x: posX, y: posY };
				// Move the element
				element.style.left = posX + 'px';
				element.style.top = posY + 'px';
				// Call any "drag" events
				self._raiseEvent('drag', item, e2);
				return stopEvent(e2);
			})
		);
		// Bind the release event
		tempEvents.push(
			Events.bind(document, events.end, function(e2) {
				// Unbind all sub-events
				for (var i = 0, c = tempEvents.length; i < c; i++) {
					Events.unbind(tempEvents[i]);
				}
				// Allow text selection again
				textSelection.enable();
				// Clean up...
				removeClass(element, dragClass);
				// Call any "dragend" events
				self._raiseEvent('dragend', item, e2);
				// Handle the item release
				self._releaseItem(
					new GridMenuEvent('release', e2, {
						item: item,
						startX: startX,
						startY: startY,
						boundingBox: {
							x: { min: minX, max: maxX },
							y: { min: minY, max: maxY }
						},
						pos: lastPosition
					})
				);
				return stopEvent(e2);
			})
		);
		// Avoid text selection problems
		textSelection.disable(element);
		// Call any "dragstart" events
		self._raiseEvent('dragstart', item, e);
		return stopEvent(e);
	};
	
	/**
	 * When an item is being dragged and gets released, this is called
	 *
	 * @access  public
	 * @param   object    the GridMenuEvent object
	 * @return  void
	 */
	GridMenu.prototype._releaseItem = function(e) {
		// First, we must determine where the mouse was upon release. This has
		// to be done by calculating relative position based on the parent LI.
		var li = e.item.node.parentNode;
		var liPos = {
			x: getPos(li, 'left'),
			y: getPos(li, 'top')
		};
		var dropPos = {
			x: liPos.x + e.pos.x,
			y: liPos.y + e.pos.y
		};
		// Now, using the position in the grid, we need to determine the grid
		// item (row/col) that the mouse is over.
		var itemDimensions = {
			x: this._conf.width / this._conf.cols,
			y: this._conf.height / this._conf.rows
		};
		var gridCoords = {
			x: Math.floor(
				(dropPos.x + (itemDimensions.x / 2)) / itemDimensions.x
			),
			y: Math.floor(
				(dropPos.y + (itemDimensions.y / 2)) / itemDimensions.y
			)
		};
		// Do the restructuring...
		var start = this._grid._getItemPosition(e.item.id);
		var end = [gridCoords.y, gridCoords.x];
		this._grid.restructure(start, end);
		this.redraw();
	};

// ------------------------------------------------------------------
//  Grid Constructor
	
	/**
	 * Grid Constructor
	 *
	 * @access  private
	 * @param   object    the GridMenu object
	 */
	function Grid(parent) {
		// Store the data
		var rows, cols, items
		this._parent = parent;
		this._cols = cols = parent._conf.cols;
		this._rows = rows = parent._conf.rows;
		this._items = items = parent._conf.items;
		// Check the dimensions of the grid
		var itemCount = items.length;
		if (itemCount > (cols * rows)) {
			throw 'Grid dimensions not large enough for the item set given'
		}
		// Build the grid structure
		this._buildLayout(range(items.length));
	}
	
	/**
	 * Move/remove an item from the grid, then restructure the layout
	 * based on the given strategy
	 *
	 * @access  public
	 * @param   array     the starting position
	 * @param   array     the ending position
	 * @return  void
	 */
	Grid.prototype.restructure = function(startPos, endPos) {
		if (startPos) {
			if (! this._positionExists(startPos)) {
				throw 'Start position does not exist in the grid layout';
			}
			if (endPos && ! this._positionExists(endPos)) {
				throw 'End position does not exist in the grid layout';
			}
		} else if (endPos) {
			throw 'Cannot give an end position without a start position';
		}
		var strategy = Grid._strategies[this._parent._conf.strategy];
		if (startPos[0] === endPos[0] && startPos[1] === endPos[1]) {
			return strategy.call(this);
		} else {
			return strategy.apply(this, arguments);
		}
	};
	
	/**
	 * Build and store a 2D layout from a single item list
	 *
	 * @access  public
	 * @param   array     the list of items
	 * @return  void
	 */
	Grid.prototype._buildLayout = function(items) {
		var rows = this._rows;
		var cols = this._cols;
		this._layout = [ ];
		for (var row = 0, item = 0; row < rows; row++) {
			this._layout.push([ ]);
			for (var col = 0; col < cols; col++, item++) {
				this._layout[row].push(items[item]);
			}
		}
	};
	
	/**
	 * Test if a given position exists in the layout
	 *
	 * @access  public
	 * @param   array     the position ([col, row])
	 * @return  boolean
	 */
	Grid.prototype._positionExists = function(pos) {
		return (this._layout[pos[0]] && this._layout[pos[0]][pos[1]] !== void(0));
	};
	
	/**
	 * Return a simple, in-order list of the layout
	 *
	 * @access  public
	 * @return  array
	 */
	Grid.prototype._buildItemList = function() {
		var list = [ ];
		for (var row = 0; row < this._rows; row++) {
			for (var col = 0; col < this._cols; col++) {
				list.push(this._layout[row][col]);
			}
		}
		return list;
	};
	
	/**
	 * Gets the item index for a given position
	 *
	 * @access  public
	 * @param   array     the position to fetch
	 * @return  number
	 */
	Grid.prototype._itemAtPosition = function(pos) {
		return this._layout[pos[0]][pos[1]];
	};

	/**
	 * Get the position of a given item in the layout
	 *
	 * @access  public
	 * @param   object    the item
	 * @return  array
	 */
	Grid.prototype._getItemPosition = function(item) {
		for (var row = 0; row < this._rows; row++) {
			for (var col = 0; col < this._cols; col++) {
				if (this._layout[row][col] === item) {
					return [row, col];
				}
			}
		}
	};
	
// ------------------------------------------------------------------
//  Grid restructuring strategies
	
	Grid._strategies = {
		
		/**
		 * Forces strict alignment collapsed to the beginning
		 *
		 * @access  public
		 * @param   array     starting position
		 * @param   array     ending position
		 * @return  array
		 */
		alignStrict: function(start, end) {
			var items = this._buildItemList();
			var startIndex, endIndex;
			// Handle any move/remove
			if (start) {
				start = this._itemAtPosition(start);
				startIndex = arrayIndexOf(items, start);
				if (end) {
					end = this._itemAtPosition(end);
					endIndex = arrayIndexOf(items, end);
					if (startIndex > endIndex) {
						startIndex++;
					} else {
						endIndex++;
					}
					items.splice(endIndex, 0, start);
					items.splice(startIndex, 1);
				} else {
					items.splice(endIndex, 1, null);
				}
			}
			// Collapse the list and rebuild the layout
			var result = [ ];
			for (var i = 0, c = items.length; i < c; i++) {
				if (typeof items[i] === 'number') {
					result.push(items[i]);
				}
			}
			this._buildLayout(result);
		}
		
	};
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	

// ------------------------------------------------------------------
//  Selection Event Object
	
	function GridMenuEvent(type, original, properties) {
		this.type = type;
		this.originalEvent = original;
		this.altKey = original.altKey || false;
		this.ctrlKey = original.ctrlKey || false;
		this.shiftKey = original.shiftKey || false;
		// Merge in extra properties
		if (isObject(properties)) {
			merge(this, properties);
		}
	}

// ------------------------------------------------------------------
//  Events Utility
	
	/**
	 * A namespace with functions for event binding
	 *
	 * Example:
	 *
	 *   Bind
	 *    var evt = Events.bind(obj, 'event', function() { ... });
	 *
	 *   Unbind
	 *    Events.unbind(evt);
	 *     -OR-
	 *    evt.unbind();
	 *
	 * @access  private
	 */
	var Events = (function() {

		var

		// Bind an event
		bindEvent = (function() {
			if (document.addEventListener) {
				return function(obj, event, func) {
					obj.addEventListener(event, func, false);
				};
			} else if (document.attachEvent) {
				return function(obj, event, func) {
					obj.attachEvent('on' + event, func);
				};
			} else {
				return function() { };
			}
		}()),

		// Unbind an event
		unbindEvent = (function() {
			if (document.removeEventListener) {
				return function(obj, event, func) {
					obj.removeEventListener(event, func, false);
				};
			} else if (document.detachEvent) {
				return function(obj, event, func) {
					obj.detachEvent('on' + event, func);
				};
			} else {
				return function() { };
			}
		}());
	
		// Build the return value
		return {
			bind: function(obj, event, func) {
				var oldFunc = (func === false) ? function(e) {
					return stopEvent(e);
				} : func;
				func = function(e) {
					return oldFunc.call(obj, e || window.event);
				};
				bindEvent(obj, event, func);
				var ret = function() {
					unbindEvent(obj, event, func);
				};
				ret.unbind = function() {ret();};
				return ret;
			},
			unbind: function(unbinder) {
				unbinder();
			}
		};

	}());

// ------------------------------------------------------------------
//  Helpers
	
	/**
	 * Does a simple, shallow object merge
	 *
	 * @access  private
	 * @param   object    the object merged on to
	 * @param   object    the object(s) being borrowed from ...
	 * @return  object
	 */
	function merge(/* host, donor[, donor ... ] */) {
		var result = arguments[0];
		for (var i = 1, c = arguments.length; i < c; i++) {
			for (var prop in arguments[i]) {
				if (arguments[i].hasOwnProperty(prop)) {
					result[prop] = arguments[i][prop];
				}
			}
		}
		return result;
	}
	
	/**
	 * Remove a value from an array
	 *
	 * @access  private
	 * @param   array     the array to remove from
	 * @param   number    the value to remove
	 * @param   mixed     an optional value to use as a replacement
	 * @return  array
	 */
	function arrayRemove(arr, value, replace) {
		var result = [ ];
		for (var i = 0, c = arr.length; i < c; i++) {
			if (arr[i] === value) {
				if (arguments.length > 2) {
					result.push(replace);
				}
			} else {
				result.push(arr[i]);
			}
		}
		return result;
	}
	
	/**
	 * Creates DOM elements of a given tag, optionally with the given attributes
	 *
	 * @access  private
	 * @param   string    the element tag name
	 * @param   object    attributes to assign
	 * @return  object
	 */
	function createElement(tag, attrs) {
		var elem, parentNode;
		// This whole if clause is optional; it is a work-around for a
		// bug in IE where the name attribute cannot be dynamically set.
		// If you don't need it for your code, feel free to remove it.
		if (attrs && attrs.hasOwnProperty('name')) {
			try {
				// This should only actually work in IE<10
				elem = document.createElement('<' + tag + ' name="' + attrs.name + '" />');
				// We no longer need to set the name
				delete attrs.name;
			} catch (e) { }
		}
		// This is the normal way of creating elements, used in all cases
		// except the one listed in the if above
		if (! elem) {
			elem = document.createElement(tag);
		}
		// Make sure the element is actually part of the DOM (some attributes
		// don't seem to apply correctly in some cases when the element is not
		// in the document).
		document.body.appendChild(elem);
		// Add attributes to the element as needed
		if (attrs) {
			for (var attr in attrs) {
				if (attrs.hasOwnProperty(attr)) {
					switch (attr) {
						// Handle an object of style values
						case 'style':
							var style = attrs[attr];
							for (var prop in style) {
								elem.style[prop] = style[prop];
							}
						break;
						// If a parentNode value is given, we will append the element
						// to this node before returning it
						case 'parentNode':
							parentNode = attrs[attr];
						break;
						// This is a special case used for injecting content
						case '_contents':
							if (typeof attrs[attr] === 'object' && attrs[attr]) {
								elem.appendChild(attrs[attr]);
							} else {
								elem.innerHTML = attrs[attr];
							}
						break;
						// For everything else, just add an attribute
						default:
							elem[attr] = attrs[attr];
						break;
					}
				}
			}
		}
		// If a parentNode was specified, use it now
		document.body.removeChild(elem);
		if (parentNode) {
			parentNode.appendChild(elem);
		}
		// All done now
		return elem;
	}
	
	/**
	 * Generate an object id
	 *
	 * @access  private
	 * @return  number
	 */
	var objectId = (function() {
		var _nextObjectId = 1;
		return function() {
			return _nextObjectId++;
		};
	}());
	
	// Array Remove - By John Resig (MIT Licensed)
	function arrayRemoveByIndex(array, from, to) {
		var rest = array.slice((to || from) + 1 || array.length);
		array.length = from < 0 ? array.length + from : from;
		return array.push.apply(array, rest);
	}

	// Get the position of an element
	function getPos(elem, from) {
		var pos = parseFloat(getStyle(elem, from));
		return (isNaN(pos) ? 0 : pos);
	}

	// Get a style property from an element
	function getStyle(elem, prop) {
		if (elem.currentStyle) {
			return elem.currentStyle[prop];
		} else if (window.getComputedStyle) {
			return document.defaultView.getComputedStyle(elem, null).getPropertyValue(prop);
		} else if (elem.style) {
			return elem.style[prop];
		}
	}

	// Get the dimensions of the window
	function getWindowSize() {
		return {
			x: window.innerWidth || document.documentElement.clientWidth || body().clientWidth,
			y: window.innerHeight || document.documentElement.clientHeight || body().clientHeight
		};
	}

	// Stop an event
	function stopEvent(evt) {
		if (evt.preventDefault) {
			evt.preventDefault();
		}
		if (evt.stopPropagation) {
			evt.stopPropagation();
		}
		evt.returnValue = false;
		return false;
	}

	// Regular expressions for matching classnames
	var cnRegexes = { };

	// Remove a class from an element
	function removeClass(elem, cn) {
		if (! cnRegexes[cn]) {
			cnRegexes[cn] = new RegExp('(^|\\s)+' + cn + '(\\s|$)+');
		}
		elem.className = elem.className.replace(cnRegexes[cn], ' ');
	}

	// Add a class to an element
	function addClass(elem, cn) {
		removeClass(elem, cn);
		elem.className += ' ' + cn;
	}

	// Check for a non-null object
	function isObject(value) {
		return !! (value && typeof value === 'object');
	}

	/**
	 * A stackable function
	 *
	 * @access  private
	 * @param   function  an initial function
	 * @return  function
	 */
	function Callstack(func) {
		var stack = [ ];
		var result = function() {
			var ret;
			for (var i = 0, c = stack.length; i < c; i++) {
				ret = stack[i].apply(this, arguments);
			}
			return ret;
		};
		result.push = function() {
			stack.push.apply(stack, arguments);
		};
		result.remove = function() {
			var args = Array.prototype.slice.call(arguments);
			var result = [ ];
			OUTER: for (var i = 0, c1 = stack.length; i < c1; i++) {
				for (var j = 0, c2 = args.length; j < c2; j++) {
					if (stack[i] === args[j]) {
						continue OUTER;
					}
				}
				result.push(stack[i]);
			}
			stack = result;
		};
		if (typeof func === 'function') {
			stack.push(func);
		}
		return result;
	}
	
	/**
	 * Basic interface for enabling/disabling text selection
	 *
	 * @access  private
	 */
	var textSelection = (function() {
		var tempEvents = null;
		return {
			enable: function() {
				if (tempEvents) {
					for (var i = 0, c = tempEvents.length; i < c; i++) {
						Events.unbind(tempEvents[i]);
					}
					tempEvents = null;
				}
			},
			disable: function(element) {
				if (! tempEvents) {
					document.body.focus();
					tempEvents = [
						Events.bind(document, 'selectstart', false),
						Events.bind(element, 'dragstart', false),
						Events.bind(document, events.move, function() {
							document.body.focus();
						})
					];
				}
			}
		};
	}());
	
	/**
	 * A shim for Array.prototype.indexOf
	 *
	 * @access  private
	 * @param   array     the array to test
	 * @param   mixed     what to look for
	 * @return  number
	 */
	var arrayIndexOf = (Array.prototype.indexOf ?
		function(arr, item) {
			return arr.indexOf(item);
		} :
		function(arr, item) {
			for (var i = 0, c = arr.length; i < c; i++) {
				if (arr[i] === item) {
					return i;
				}
			}
			return -1;
		}
	);

	/**
	 * Create an array of numbers in sequence
	 *
	 * @access  public
	 * @param   number    the length to build the array
	 * @return  void
	 */
	function range(length) {
		var result = [ ];
		for (var i = 0; i < length; i++) {
			result.push(i);
		}
		return result;
	}





















// ------------------------------------------------------------------
//  Expose
	
	return GridMenu;
	
}());
