// ------------------------------------------------------------------
//  Grid Constructor
	
	/**
	 * Grid Constructor
	 *
	 * @access  private
	 * @param   object    the GridMenu object
	 */
	function Grid(parent) {
		// Store the data
		var rows, cols, items
		this._parent = parent;
		this._cols = cols = parent._conf.cols;
		this._rows = rows = parent._conf.rows;
		this._items = items = parent._conf.items;
		// Check the dimensions of the grid
		var itemCount = items.length;
		if (itemCount > (cols * rows)) {
			throw 'Grid dimensions not large enough for the item set given'
		}
		// Build the grid structure
		this._buildLayout(range(items.length));
	}
	
	/**
	 * Move/remove an item from the grid, then restructure the layout
	 * based on the given strategy
	 *
	 * @access  public
	 * @param   array     the starting position
	 * @param   array     the ending position
	 * @return  void
	 */
	Grid.prototype.restructure = function(startPos, endPos) {
		if (startPos) {
			if (! this._positionExists(startPos)) {
				throw 'Start position does not exist in the grid layout';
			}
			if (endPos && ! this._positionExists(endPos)) {
				throw 'End position does not exist in the grid layout';
			}
		} else if (endPos) {
			throw 'Cannot give an end position without a start position';
		}
		var strategy = Grid._strategies[this._parent._conf.strategy];
		if (startPos[0] === endPos[0] && startPos[1] === endPos[1]) {
			return strategy.call(this);
		} else {
			return strategy.apply(this, arguments);
		}
	};
	
	/**
	 * Build and store a 2D layout from a single item list
	 *
	 * @access  public
	 * @param   array     the list of items
	 * @return  void
	 */
	Grid.prototype._buildLayout = function(items) {
		var rows = this._rows;
		var cols = this._cols;
		this._layout = [ ];
		for (var row = 0, item = 0; row < rows; row++) {
			this._layout.push([ ]);
			for (var col = 0; col < cols; col++, item++) {
				this._layout[row].push(items[item]);
			}
		}
	};
	
	/**
	 * Test if a given position exists in the layout
	 *
	 * @access  public
	 * @param   array     the position ([col, row])
	 * @return  boolean
	 */
	Grid.prototype._positionExists = function(pos) {
		return (this._layout[pos[0]] && this._layout[pos[0]][pos[1]] !== void(0));
	};
	
	/**
	 * Return a simple, in-order list of the layout
	 *
	 * @access  public
	 * @return  array
	 */
	Grid.prototype._buildItemList = function() {
		var list = [ ];
		for (var row = 0; row < this._rows; row++) {
			for (var col = 0; col < this._cols; col++) {
				list.push(this._layout[row][col]);
			}
		}
		return list;
	};
	
	/**
	 * Gets the item index for a given position
	 *
	 * @access  public
	 * @param   array     the position to fetch
	 * @return  number
	 */
	Grid.prototype._itemAtPosition = function(pos) {
		return this._layout[pos[0]][pos[1]];
	};

	/**
	 * Get the position of a given item in the layout
	 *
	 * @access  public
	 * @param   object    the item
	 * @return  array
	 */
	Grid.prototype._getItemPosition = function(item) {
		for (var row = 0; row < this._rows; row++) {
			for (var col = 0; col < this._cols; col++) {
				if (this._layout[row][col] === item) {
					return [row, col];
				}
			}
		}
	};
	
// ------------------------------------------------------------------
//  Grid restructuring strategies
	
	Grid._strategies = {
		
		/**
		 * Forces strict alignment collapsed to the beginning
		 *
		 * @access  public
		 * @param   array     starting position
		 * @param   array     ending position
		 * @return  array
		 */
		alignStrict: function(start, end) {
			var items = this._buildItemList();
			var startIndex, endIndex;
			// Handle any move/remove
			if (start) {
				start = this._itemAtPosition(start);
				startIndex = arrayIndexOf(items, start);
				if (end) {
					end = this._itemAtPosition(end);
					endIndex = arrayIndexOf(items, end);
					if (startIndex > endIndex) {
						startIndex++;
					} else {
						endIndex++;
					}
					items.splice(endIndex, 0, start);
					items.splice(startIndex, 1);
				} else {
					items.splice(endIndex, 1, null);
				}
			}
			// Collapse the list and rebuild the layout
			var result = [ ];
			for (var i = 0, c = items.length; i < c; i++) {
				if (typeof items[i] === 'number') {
					result.push(items[i]);
				}
			}
			this._buildLayout(result);
		}
		
	};
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	

