// ------------------------------------------------------------------
//  GridMenu Constructor

	/**
	 * GridMenu Constructor
	 *
	 * @access  public
	 * @param   object    configuration options
	 */
	function GridMenu(conf) {
		this._objectId = objectId();
		this._conf = merge({ }, _defaultConfig, conf);
		// Make sure the strategy exists
		if (typeof Grid._strategies[this._conf.strategy] !== 'function') {
			throw 'Invalid grid restructure strategy "' + this._conf.strategy + '"';
		}
		// Check/clone the item objects
		var items = this._conf.items;
		for (var i = 0, c = items.length; i < c; i++) {
			if (! isObject(items[i])) {
				items[i] = { contents: items[i] };
			}
			items[i] = merge({ id: i }, _defaultItem, items[i]);
		}
		// Build the layout grid
		this._grid = new Grid(this);
		// Build the DOM structure
		this._create();
	}
	
	/**
	 * Make any changes in the layout object
	 *
	 * @access  public
	 * @return  void
	 */
	GridMenu.prototype.redraw = function() {
		var items = this._conf.items;
		// Pull all the item elements
		for (var i = 0, c = items.length; i < c; i++) {
			items[i].node.parentNode.removeChild(items[i].node);
		}
		// Put the items back in in the correct order
		var layout = this._grid._buildItemList();
		var lis = this.domNode.getElementsByTagName('li');
		for (var item, i = 0, c = layout.length; i < c; i++) {
			if (typeof layout[i] !== 'number') {continue;}
			item = items[layout[i]];
			item.node.style.top = '0px';
			item.node.style.left = '0px';
			lis[i].appendChild(item.node);
		}
	};
	
	/**
	 * Create the item elements and the DOM structure
	 *
	 * @access  public
	 * @return  void
	 */
	GridMenu.prototype._create = function() {
		// We need this in the event binding later...
		var self = this;
		// Create the root element
		var root = createElement('ul', {
			id: 'grid-menu-' + this._objectId,
			style: {
				position: 'relative',
				width: this._conf.width + 'px',
				height: this._conf.height + 'px'
			}
		});
		// Prepare to build the items
		var items = this._conf.items;
		var itemDimensions = {
			x: this._conf.width / this._conf.cols,
			y: this._conf.height / this._conf.rows
		};
		var currentLi;
		for (var row = 0, item = 0; row < this._conf.rows; row++) {
			for (var col = 0; col < this._conf.cols; col++, item++) {
				// Build the item's LI
				currentLi = root.appendChild(
					createElement('li', {
						style: {
							position: 'absolute',
							width: itemDimensions.x + 'px',
							height: itemDimensions.y + 'px',
							top: (itemDimensions.y * row) + 'px',
							left: (itemDimensions.x * col) + 'px'
						}
					})
				);
				// Build the item's DIV
				items[item].node = currentLi.appendChild(
					createElement('div', {
						_contents: items[item].contents,
						className: 'grid-menu-item',
						id: 'grid-menu-item-' + objectId(),
						style: {
							top: '0px',
							left: '0px',
							position: 'relative',
							width: itemDimensions.x + 'px',
							height: itemDimensions.y + 'px'
						}
					})
				);
				// Create a closure to hold on to a reference to the node
				(function(element, item) {
					// Bind the initializing event
					element._menuEvent = Events.bind(element, events.start, function(e) {
						// Make sure that if this is a mouse event, that the left mouse
						// button is what was clicked. (this structure is kind of ugly
						// with 3 nested ifs, but its the best way I can think to write
						// it, so just live with it...)
						if (e.type === 'mousedown') {
							if (window.event === e) {
								if (e.button === 1) {return false;}
							} else if (e.button === 0) {return false;}
						}
						var releaseTimer;
						var releaseEvent;
						// Releases both the event and timer
						function release() {
							releaseEvent();
							clearTimeout(releaseTimer);
							textSelection.enable();
						}
						// Start the drag timer/flag
						releaseTimer = setTimeout(function() {
							// If the strategy value is "none", don't allow rearrangement
							if (self._conf.strategy !== 'none') {
								release();
								self._dragItem(item, e);
							}
						}, self._conf.dragDelay);
						// Bind the release event
						releaseEvent = Events.bind(document, events.end, function(e) {
							release();
							self._raiseEvent('select', item, e);
						});
						// Avoid text selection problems
						textSelection.disable(element);
					});
				}(items[item].node, items[item]));
			}
		}
		// Expose the top-level DOM node
		this.domNode = root;
	};
	
	/**
	 * This function is called when an item is selected (quick tap)
	 *
	 * @access  public
	 * @param   string    the event type
	 * @param   object    the item object
	 * @param   object    the event object
	 * @return  void
	 */
	GridMenu.prototype._raiseEvent = function(type, item, event) {
		if (typeof item['on' + type] === 'function') {
			item['on' + type].call(this,
				new GridMenuEvent(type, event, { item: item })
			);
		}
	};
	
	/**
	 * This initializes the item dragging
	 *
	 * @access  public
	 * @param   object    the item object
	 * @param   object    the event object
	 * @return  void
	 */
	GridMenu.prototype._dragItem = function(item, e) {
		var self = this;
		var element = item.node;
		// Add the dragging class to the DIV
		addClass(element, dragClass);
		// Store a reference to the current position
		var startX = getPos(element, 'left');
		var startY = getPos(element, 'top');
		// Determine the bounding box
		var liNode = element.parentNode;
		var minX = -getPos(liNode, 'left');
		var maxX = minX + self._conf.width;
		var minY = -getPos(liNode, 'top');
		var maxY = minY + self._conf.height;
		// We need to know the last position when we release
		var lastPosition = null;
		// Bind the drag event
		var tempEvents = [ ];
		tempEvents.push(
			Events.bind(document, events.move, function(e2) {
				// Find all needed offsets
				var offsetX = e2.clientX - e.clientX;
				var offsetY = e2.clientY - e.clientY;
				var offsetWidth = element.offsetWidth;
				var offsetHeight = element.offsetHeight;
				// Find the new positions
				var posX = startX + offsetX;
				var posY = startY + offsetY;
				// Enforce the bounding box
				posX = Math.max(minX, Math.min(maxX - offsetWidth, posX));
				posY = Math.max(minY, Math.min(maxY - offsetHeight, posY));
				// Store the position
				lastPosition = { x: posX, y: posY };
				// Move the element
				element.style.left = posX + 'px';
				element.style.top = posY + 'px';
				// Call any "drag" events
				self._raiseEvent('drag', item, e2);
				return stopEvent(e2);
			})
		);
		// Bind the release event
		tempEvents.push(
			Events.bind(document, events.end, function(e2) {
				// Unbind all sub-events
				for (var i = 0, c = tempEvents.length; i < c; i++) {
					Events.unbind(tempEvents[i]);
				}
				// Allow text selection again
				textSelection.enable();
				// Clean up...
				removeClass(element, dragClass);
				// Call any "dragend" events
				self._raiseEvent('dragend', item, e2);
				// Handle the item release
				self._releaseItem(
					new GridMenuEvent('release', e2, {
						item: item,
						startX: startX,
						startY: startY,
						boundingBox: {
							x: { min: minX, max: maxX },
							y: { min: minY, max: maxY }
						},
						pos: lastPosition
					})
				);
				return stopEvent(e2);
			})
		);
		// Avoid text selection problems
		textSelection.disable(element);
		// Call any "dragstart" events
		self._raiseEvent('dragstart', item, e);
		return stopEvent(e);
	};
	
	/**
	 * When an item is being dragged and gets released, this is called
	 *
	 * @access  public
	 * @param   object    the GridMenuEvent object
	 * @return  void
	 */
	GridMenu.prototype._releaseItem = function(e) {
		// First, we must determine where the mouse was upon release. This has
		// to be done by calculating relative position based on the parent LI.
		var li = e.item.node.parentNode;
		var liPos = {
			x: getPos(li, 'left'),
			y: getPos(li, 'top')
		};
		var dropPos = {
			x: liPos.x + e.pos.x,
			y: liPos.y + e.pos.y
		};
		// Now, using the position in the grid, we need to determine the grid
		// item (row/col) that the mouse is over.
		var itemDimensions = {
			x: this._conf.width / this._conf.cols,
			y: this._conf.height / this._conf.rows
		};
		var gridCoords = {
			x: Math.floor(
				(dropPos.x + (itemDimensions.x / 2)) / itemDimensions.x
			),
			y: Math.floor(
				(dropPos.y + (itemDimensions.y / 2)) / itemDimensions.y
			)
		};
		// Do the restructuring...
		var start = this._grid._getItemPosition(e.item.id);
		var end = [gridCoords.y, gridCoords.x];
		this._grid.restructure(start, end);
		this.redraw();
	};

