// ------------------------------------------------------------------
//  Helpers
	
	/**
	 * Does a simple, shallow object merge
	 *
	 * @access  private
	 * @param   object    the object merged on to
	 * @param   object    the object(s) being borrowed from ...
	 * @return  object
	 */
	function merge(/* host, donor[, donor ... ] */) {
		var result = arguments[0];
		for (var i = 1, c = arguments.length; i < c; i++) {
			for (var prop in arguments[i]) {
				if (arguments[i].hasOwnProperty(prop)) {
					result[prop] = arguments[i][prop];
				}
			}
		}
		return result;
	}
	
	/**
	 * Remove a value from an array
	 *
	 * @access  private
	 * @param   array     the array to remove from
	 * @param   number    the value to remove
	 * @param   mixed     an optional value to use as a replacement
	 * @return  array
	 */
	function arrayRemove(arr, value, replace) {
		var result = [ ];
		for (var i = 0, c = arr.length; i < c; i++) {
			if (arr[i] === value) {
				if (arguments.length > 2) {
					result.push(replace);
				}
			} else {
				result.push(arr[i]);
			}
		}
		return result;
	}
	
	/**
	 * Creates DOM elements of a given tag, optionally with the given attributes
	 *
	 * @access  private
	 * @param   string    the element tag name
	 * @param   object    attributes to assign
	 * @return  object
	 */
	function createElement(tag, attrs) {
		var elem, parentNode;
		// This whole if clause is optional; it is a work-around for a
		// bug in IE where the name attribute cannot be dynamically set.
		// If you don't need it for your code, feel free to remove it.
		if (attrs && attrs.hasOwnProperty('name')) {
			try {
				// This should only actually work in IE<10
				elem = document.createElement('<' + tag + ' name="' + attrs.name + '" />');
				// We no longer need to set the name
				delete attrs.name;
			} catch (e) { }
		}
		// This is the normal way of creating elements, used in all cases
		// except the one listed in the if above
		if (! elem) {
			elem = document.createElement(tag);
		}
		// Make sure the element is actually part of the DOM (some attributes
		// don't seem to apply correctly in some cases when the element is not
		// in the document).
		document.body.appendChild(elem);
		// Add attributes to the element as needed
		if (attrs) {
			for (var attr in attrs) {
				if (attrs.hasOwnProperty(attr)) {
					switch (attr) {
						// Handle an object of style values
						case 'style':
							var style = attrs[attr];
							for (var prop in style) {
								elem.style[prop] = style[prop];
							}
						break;
						// If a parentNode value is given, we will append the element
						// to this node before returning it
						case 'parentNode':
							parentNode = attrs[attr];
						break;
						// This is a special case used for injecting content
						case '_contents':
							if (typeof attrs[attr] === 'object' && attrs[attr]) {
								elem.appendChild(attrs[attr]);
							} else {
								elem.innerHTML = attrs[attr];
							}
						break;
						// For everything else, just add an attribute
						default:
							elem[attr] = attrs[attr];
						break;
					}
				}
			}
		}
		// If a parentNode was specified, use it now
		document.body.removeChild(elem);
		if (parentNode) {
			parentNode.appendChild(elem);
		}
		// All done now
		return elem;
	}
	
	/**
	 * Generate an object id
	 *
	 * @access  private
	 * @return  number
	 */
	var objectId = (function() {
		var _nextObjectId = 1;
		return function() {
			return _nextObjectId++;
		};
	}());
	
	// Array Remove - By John Resig (MIT Licensed)
	function arrayRemoveByIndex(array, from, to) {
		var rest = array.slice((to || from) + 1 || array.length);
		array.length = from < 0 ? array.length + from : from;
		return array.push.apply(array, rest);
	}

	// Get the position of an element
	function getPos(elem, from) {
		var pos = parseFloat(getStyle(elem, from));
		return (isNaN(pos) ? 0 : pos);
	}

	// Get a style property from an element
	function getStyle(elem, prop) {
		if (elem.currentStyle) {
			return elem.currentStyle[prop];
		} else if (window.getComputedStyle) {
			return document.defaultView.getComputedStyle(elem, null).getPropertyValue(prop);
		} else if (elem.style) {
			return elem.style[prop];
		}
	}

	// Get the dimensions of the window
	function getWindowSize() {
		return {
			x: window.innerWidth || document.documentElement.clientWidth || body().clientWidth,
			y: window.innerHeight || document.documentElement.clientHeight || body().clientHeight
		};
	}

	// Stop an event
	function stopEvent(evt) {
		if (evt.preventDefault) {
			evt.preventDefault();
		}
		if (evt.stopPropagation) {
			evt.stopPropagation();
		}
		evt.returnValue = false;
		return false;
	}

	// Regular expressions for matching classnames
	var cnRegexes = { };

	// Remove a class from an element
	function removeClass(elem, cn) {
		if (! cnRegexes[cn]) {
			cnRegexes[cn] = new RegExp('(^|\\s)+' + cn + '(\\s|$)+');
		}
		elem.className = elem.className.replace(cnRegexes[cn], ' ');
	}

	// Add a class to an element
	function addClass(elem, cn) {
		removeClass(elem, cn);
		elem.className += ' ' + cn;
	}

	// Check for a non-null object
	function isObject(value) {
		return !! (value && typeof value === 'object');
	}

	/**
	 * A stackable function
	 *
	 * @access  private
	 * @param   function  an initial function
	 * @return  function
	 */
	function Callstack(func) {
		var stack = [ ];
		var result = function() {
			var ret;
			for (var i = 0, c = stack.length; i < c; i++) {
				ret = stack[i].apply(this, arguments);
			}
			return ret;
		};
		result.push = function() {
			stack.push.apply(stack, arguments);
		};
		result.remove = function() {
			var args = Array.prototype.slice.call(arguments);
			var result = [ ];
			OUTER: for (var i = 0, c1 = stack.length; i < c1; i++) {
				for (var j = 0, c2 = args.length; j < c2; j++) {
					if (stack[i] === args[j]) {
						continue OUTER;
					}
				}
				result.push(stack[i]);
			}
			stack = result;
		};
		if (typeof func === 'function') {
			stack.push(func);
		}
		return result;
	}
	
	/**
	 * Basic interface for enabling/disabling text selection
	 *
	 * @access  private
	 */
	var textSelection = (function() {
		var tempEvents = null;
		return {
			enable: function() {
				if (tempEvents) {
					for (var i = 0, c = tempEvents.length; i < c; i++) {
						Events.unbind(tempEvents[i]);
					}
					tempEvents = null;
				}
			},
			disable: function(element) {
				if (! tempEvents) {
					document.body.focus();
					tempEvents = [
						Events.bind(document, 'selectstart', false),
						Events.bind(element, 'dragstart', false),
						Events.bind(document, events.move, function() {
							document.body.focus();
						})
					];
				}
			}
		};
	}());
	
	/**
	 * A shim for Array.prototype.indexOf
	 *
	 * @access  private
	 * @param   array     the array to test
	 * @param   mixed     what to look for
	 * @return  number
	 */
	var arrayIndexOf = (Array.prototype.indexOf ?
		function(arr, item) {
			return arr.indexOf(item);
		} :
		function(arr, item) {
			for (var i = 0, c = arr.length; i < c; i++) {
				if (arr[i] === item) {
					return i;
				}
			}
			return -1;
		}
	);

	/**
	 * Create an array of numbers in sequence
	 *
	 * @access  public
	 * @param   number    the length to build the array
	 * @return  void
	 */
	function range(length) {
		var result = [ ];
		for (var i = 0; i < length; i++) {
			result.push(i);
		}
		return result;
	}





















